@@ -428,6 +428,15 @@ class ClickHouse(Dialect):
             "INDEX",
         }
 
+        def _parse_create(self) -> exp.Create | exp.Command:
+            create = super()._parse_create()
+
+            # DATABASE in ClickHouse is the same as SCHEMA in other dialects
+            if isinstance(create, exp.Create) and create.kind == "DATABASE":
+                create.set("kind", "SCHEMA")
+
+            return create
+
         def _parse_extract(self) -> exp.Extract | exp.Anonymous:
             index = self._index
             this = self._parse_bitwise()
@@ -437,7 +446,7 @@ class ClickHouse(Dialect):
 
             # We return Anonymous here because extract and regexpExtract have different semantics,
             # so parsing extract(foo, bar) into RegexpExtract can potentially break queries. E.g.,
-            # `extract('foobar', 'b')` works, but CH crashes for `regexpExtract('foobar', 'b')`.
+            # `extract('foobar', 'b')` works, but ClickHouse crashes for `regexpExtract('foobar', 'b')`.
             #
             # TODO: can we somehow convert the former into an equivalent `regexpExtract` call?
             self._match(TokenType.COMMA)
@@ -593,7 +602,7 @@ class ClickHouse(Dialect):
 
                 if isinstance(expr, exp.Window):
                     # The window's func was parsed as Anonymous in base parser, fix its
-                    # type to be CH style CombinedAnonymousAggFunc / AnonymousAggFunc
+                    # type to be ClickHouse style CombinedAnonymousAggFunc / AnonymousAggFunc
                     expr.set("this", func)
                 elif params:
                     # Params have blocked super()._parse_function() from parsing the following window
@@ -852,6 +861,35 @@ class ClickHouse(Dialect):
             "NAMED COLLECTION",
         }
 
+        # https://github.com/ClickHouse/ClickHouse/blob/275de04b8f6bb8c9334bf8070001afe2dab0b17d/src/Functions/FunctionsConversion.cpp#L2939-L2989
+        TRY_CAST_TYPES = {
+            "DATE",
+            "DATE32",
+            "DATETIME",
+            "DATETIME64",
+            "DECIMAL32",
+            "DECIMAL64",
+            "DECIMAL128",
+            "DECIMAL256",
+            "FLOAT32",
+            "FLOAT64",
+            "INT8",
+            "INT16",
+            "INT32",
+            "INT64",
+            "INT128",
+            "INT256",
+            "IPV4",
+            "IPV6",
+            "UINT8",
+            "UINT16",
+            "UINT32",
+            "UINT64",
+            "UINT128",
+            "UINT256",
+            "UUID",
+        }
+
         def strtodate_sql(self, expression: exp.StrToDate) -> str:
             strtodate_sql = self.function_fallback_sql(expression)
 
@@ -870,6 +908,14 @@ class ClickHouse(Dialect):
 
             return super().cast_sql(expression, safe_prefix=safe_prefix)
 
+        def trycast_sql(self, expression: exp.TryCast) -> str:
+            target_type = self.sql(expression.to)
+            if target_type.upper() in self.TRY_CAST_TYPES:
+                return self.func(f"to{target_type}OrNull", expression.this)
+
+            self.unsupported(f"There is no `to<Type>OrNull` for type {target_type}.")
+            return super().cast_sql(expression)
+
         def _jsonpathsubscript_sql(self, expression: exp.JSONPathSubscript) -> str:
             this = self.json_path_part(expression.this)
             return str(int(this) + 1) if is_int(this) else this
@@ -960,7 +1006,10 @@ class ClickHouse(Dialect):
             if expression.kind in self.ON_CLUSTER_TARGETS and locations.get(
                 exp.Properties.Location.POST_NAME
             ):
-                this_name = self.sql(expression.this, "this")
+                this_name = self.sql(
+                    expression.this if isinstance(expression.this, exp.Schema) else expression,
+                    "this",
+                )
                 this_properties = " ".join(
                     [self.sql(prop) for prop in locations[exp.Properties.Location.POST_NAME]]
                 )
@@ -969,6 +1018,28 @@ class ClickHouse(Dialect):
 
             return super().createable_sql(expression, locations)
 
+        def create_sql(self, expression: exp.Create) -> str:
+            # The comment property comes last in CTAS statements, i.e. after the query
+            query = expression.expression
+            if isinstance(query, exp.Query):
+                comment_prop = expression.find(exp.SchemaCommentProperty)
+                if comment_prop:
+                    comment_prop.pop()
+                    query.replace(exp.paren(query))
+            else:
+                comment_prop = None
+
+            # ClickHouse only has DATABASEs and objects under them, eg. TABLEs, VIEWs, etc
+            if expression.kind == "SCHEMA":
+                expression.set("kind", "DATABASE")
+
+            create_sql = super().create_sql(expression)
+
+            comment_sql = self.sql(comment_prop)
+            comment_sql = f" {comment_sql}" if comment_sql else ""
+
+            return f"{create_sql}{comment_sql}"
+
         def prewhere_sql(self, expression: exp.PreWhere) -> str:
             this = self.indent(self.sql(expression, "this"))
             return f"{self.seg('PREWHERE')}{self.sep()}{this}"
