@@ -727,6 +727,7 @@ class MySQL(Dialect):
                     transforms.eliminate_semi_and_anti_joins,
                     transforms.eliminate_qualify,
                     transforms.eliminate_full_outer_join,
+                    transforms.unnest_generate_date_array_using_recursive_cte,
                 ]
             ),
             exp.StrPosition: strposition_to_locate_sql,
@@ -753,9 +754,6 @@ class MySQL(Dialect):
             exp.TsOrDsDiff: lambda self, e: self.func("DATEDIFF", e.this, e.expression),
             exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
             exp.UnixToTime: _unix_to_time_sql,
-            exp.Unnest: transforms.preprocess(
-                [transforms.unnest_generate_date_array_using_recursive_cte()]
-            ),
             exp.Week: _remove_ts_or_ds_to_date(),
             exp.WeekOfYear: _remove_ts_or_ds_to_date(rename_func("WEEKOFYEAR")),
             exp.Year: _remove_ts_or_ds_to_date(),
