@@ -863,6 +863,7 @@ class TSQL(Dialect):
                     transforms.eliminate_distinct_on,
                     transforms.eliminate_semi_and_anti_joins,
                     transforms.eliminate_qualify,
+                    transforms.unnest_generate_date_array_using_recursive_cte,
                 ]
             ),
             exp.Stddev: rename_func("STDEV"),
@@ -882,13 +883,6 @@ class TSQL(Dialect):
             exp.Trim: trim_sql,
             exp.TsOrDsAdd: date_delta_sql("DATEADD", cast=True),
             exp.TsOrDsDiff: date_delta_sql("DATEDIFF"),
-            exp.Unnest: transforms.preprocess(
-                [
-                    transforms.unnest_generate_date_array_using_recursive_cte(
-                        bubble_up_recursive_cte=True
-                    )
-                ]
-            ),
         }
 
         TRANSFORMS.pop(exp.ReturnsProperty)
