@@ -62,19 +62,37 @@ class TestTranspile(unittest.TestCase):
     def test_some(self):
         self.validate(
             "SELECT * FROM x WHERE a = SOME (SELECT 1)",
-            "SELECT * FROM x WHERE a = ANY (SELECT 1)",
+            "SELECT * FROM x WHERE a = ANY(SELECT 1)",
         )
 
     def test_leading_comma(self):
+        self.validate(
+            "SELECT a, b, c FROM (SELECT a, b, c FROM t)",
+            "SELECT\n"
+            "    a\n"
+            "    , b\n"
+            "    , c\n"
+            "FROM (\n"
+            "    SELECT\n"
+            "        a\n"
+            "        , b\n"
+            "        , c\n"
+            "    FROM t\n"
+            ")",
+            leading_comma=True,
+            pretty=True,
+            pad=4,
+            indent=4,
+        )
         self.validate(
             "SELECT FOO, BAR, BAZ",
-            "SELECT\n    FOO\n  , BAR\n  , BAZ",
+            "SELECT\n  FOO\n  , BAR\n  , BAZ",
             leading_comma=True,
             pretty=True,
         )
         self.validate(
             "SELECT FOO, /*x*/\nBAR, /*y*/\nBAZ",
-            "SELECT\n    FOO /* x */\n  , BAR /* y */\n  , BAZ",
+            "SELECT\n  FOO /* x */\n  , BAR /* y */\n  , BAZ",
             leading_comma=True,
             pretty=True,
         )
@@ -93,6 +111,10 @@ class TestTranspile(unittest.TestCase):
         self.validate("SELECT a\r\nFROM b", "SELECT a FROM b")
 
     def test_comments(self):
+        self.validate(
+            "select /* asfd /* asdf */ asdf */ 1",
+            "/* asfd /* asdf */ asdf */ SELECT 1",
+        )
         self.validate(
             "SELECT c /* foo */ AS alias",
             "SELECT c AS alias /* foo */",
@@ -105,6 +127,10 @@ class TestTranspile(unittest.TestCase):
             "SELECT * FROM t1\n/*x*/\nUNION ALL SELECT * FROM t2",
             "SELECT * FROM t1 /* x */ UNION ALL SELECT * FROM t2",
         )
+        self.validate(
+            "/* comment */ SELECT * FROM a UNION SELECT * FROM b",
+            "/* comment */ SELECT * FROM a UNION SELECT * FROM b",
+        )
         self.validate(
             "SELECT * FROM t1\n/*x*/\nINTERSECT ALL SELECT * FROM t2",
             "SELECT * FROM t1 /* x */ INTERSECT ALL SELECT * FROM t2",
@@ -275,7 +301,7 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
 FROM b
 /* where */
 WHERE
-  foo /* comment 1 */ AND bar AND bla /* comment 2 */""",
+  foo AND /* comment 1 */ bar AND /* comment 2 */ bla""",
             pretty=True,
         )
         self.validate(
@@ -428,7 +454,8 @@ FROM dw_1_dw_1_1.exactonline_2.transactionlines""",
             """SELECT
   'hotel1' AS hotel,
   *
-FROM dw_1_dw_1_1.exactonline_1.transactionlines /*
+FROM dw_1_dw_1_1.exactonline_1.transactionlines
+/*
     UNION ALL
     SELECT
       'Thon Partner Hotel Jølster' AS hotel,
@@ -479,6 +506,65 @@ SELECT
 FROM base""",
             pretty=True,
         )
+        self.validate(
+            """-- comment
+SOME_FUNC(arg IGNORE NULLS)
+  OVER (PARTITION BY foo ORDER BY bla) AS col""",
+            "SOME_FUNC(arg IGNORE NULLS) OVER (PARTITION BY foo ORDER BY bla) AS col /* comment */",
+            pretty=True,
+        )
+        self.validate(
+            """
+            SELECT *
+            FROM x
+            INNER JOIN y
+            -- inner join z
+            LEFT JOIN z using (id)
+            using (id)
+            """,
+            """SELECT
+  *
+FROM x
+INNER JOIN y
+  /* inner join z */
+  LEFT JOIN z
+    USING (id)
+  USING (id)""",
+            pretty=True,
+        )
+        self.validate(
+            """with x as (
+  SELECT *
+  /*
+NOTE: LEFT JOIN because blah blah blah
+  */
+  FROM a
+)
+select * from x""",
+            """WITH x AS (
+  SELECT
+    *
+  /*
+NOTE: LEFT JOIN because blah blah blah
+  */
+  FROM a
+)
+SELECT
+  *
+FROM x""",
+            pretty=True,
+        )
+
+        self.validate(
+            """SELECT X FROM  catalog.db.table WHERE Y
+        --
+        AND Z""",
+            """SELECT X FROM catalog.db.table WHERE Y AND Z""",
+        )
+        self.validate(
+            """with a as /* comment */ ( select * from b) select * from a""",
+            """WITH a AS (SELECT * FROM b) /* comment */ SELECT * FROM a""",
+        )
 
     def test_types(self):
         self.validate("INT 1", "CAST(1 AS INT)")
@@ -507,24 +593,24 @@ FROM base""",
     def test_extract(self):
         self.validate(
             "EXTRACT(day FROM '2020-01-01'::TIMESTAMP)",
-            "EXTRACT(day FROM CAST('2020-01-01' AS TIMESTAMP))",
+            "EXTRACT(DAY FROM CAST('2020-01-01' AS TIMESTAMP))",
         )
         self.validate(
             "EXTRACT(timezone FROM '2020-01-01'::TIMESTAMP)",
-            "EXTRACT(timezone FROM CAST('2020-01-01' AS TIMESTAMP))",
+            "EXTRACT(TIMEZONE FROM CAST('2020-01-01' AS TIMESTAMP))",
         )
         self.validate(
             "EXTRACT(year FROM '2020-01-01'::TIMESTAMP WITH TIME ZONE)",
-            "EXTRACT(year FROM CAST('2020-01-01' AS TIMESTAMPTZ))",
+            "EXTRACT(YEAR FROM CAST('2020-01-01' AS TIMESTAMPTZ))",
         )
         self.validate(
             "extract(month from '2021-01-31'::timestamp without time zone)",
-            "EXTRACT(month FROM CAST('2021-01-31' AS TIMESTAMP))",
+            "EXTRACT(MONTH FROM CAST('2021-01-31' AS TIMESTAMP))",
         )
-        self.validate("extract(week from current_date + 2)", "EXTRACT(week FROM CURRENT_DATE + 2)")
+        self.validate("extract(week from current_date + 2)", "EXTRACT(WEEK FROM CURRENT_DATE + 2)")
         self.validate(
             "EXTRACT(minute FROM datetime1 - datetime2)",
-            "EXTRACT(minute FROM datetime1 - datetime2)",
+            "EXTRACT(MINUTE FROM datetime1 - datetime2)",
         )
 
     def test_if(self):
@@ -676,7 +762,11 @@ FROM base""",
         )
 
         self.validate("STR_TO_TIME('x', 'y')", "DATE_PARSE('x', 'y')", write="presto")
-        self.validate("STR_TO_UNIX('x', 'y')", "TO_UNIXTIME(DATE_PARSE('x', 'y'))", write="presto")
+        self.validate(
+            "STR_TO_UNIX('x', 'y')",
+            "TO_UNIXTIME(COALESCE(TRY(DATE_PARSE(CAST('x' AS VARCHAR), 'y')), PARSE_DATETIME(DATE_FORMAT(CAST('x' AS TIMESTAMP), 'y'), 'y')))",
+            write="presto",
+        )
         self.validate("TIME_TO_STR(x, 'y')", "DATE_FORMAT(x, 'y')", write="presto")
         self.validate("TIME_TO_UNIX(x)", "TO_UNIXTIME(x)", write="presto")
         self.validate(
@@ -714,7 +804,10 @@ FROM base""",
 
             self.validate("x[x - 1]", "x[x - 1]", write="presto", identity=False)
             self.validate(
-                "x[array_size(y) - 1]", "x[CARDINALITY(y) - 1 + 1]", write="presto", identity=False
+                "x[array_size(y) - 1]",
+                "x[(CARDINALITY(y) - 1) + 1]",
+                write="presto",
+                identity=False,
             )
             self.validate("x[3 - 1]", "x[3]", write="presto", identity=False)
             self.validate("MAP(a, b)[0]", "MAP(a, b)[0]", write="presto", identity=False)
@@ -722,10 +815,10 @@ FROM base""",
             self.assertEqual(
                 cm.output,
                 [
-                    "WARNING:sqlglot:Applying array index offset (1)",
-                    "WARNING:sqlglot:Applying array index offset (-1)",
-                    "WARNING:sqlglot:Applying array index offset (1)",
-                    "WARNING:sqlglot:Applying array index offset (1)",
+                    "INFO:sqlglot:Applying array index offset (1)",
+                    "INFO:sqlglot:Applying array index offset (-1)",
+                    "INFO:sqlglot:Applying array index offset (1)",
+                    "INFO:sqlglot:Applying array index offset (1)",
                 ],
             )
 
@@ -752,13 +845,11 @@ FROM base""",
             "ALTER TABLE table1 RENAME COLUMN c1 TO c2, c2 TO c3",
             "ALTER TABLE table1 RENAME COLUMN c1 c2",
             "ALTER TYPE electronic_mail RENAME TO email",
-            "ALTER VIEW foo ALTER COLUMN bla SET DEFAULT 'NOT SET'",
             "ALTER schema doo",
             "ANALYZE a.y",
             "CALL catalog.system.iceberg_procedure_name(named_arg_1 => 'arg_1', named_arg_2 => 'arg_2')",
             "COMMENT ON ACCESS METHOD gin IS 'GIN index access method'",
             "CREATE OR REPLACE STAGE",
-            "CREATE SET GLOBAL TEMPORARY TABLE a, NO BEFORE JOURNAL, NO AFTER JOURNAL, MINIMUM DATABLOCKSIZE, BLOCKCOMPRESSION=NEVER (a INT)",
             "EXECUTE statement",
             "EXPLAIN SELECT * FROM x",
             "GRANT INSERT ON foo TO bla",
@@ -768,7 +859,6 @@ FROM base""",
             "SET -v",
             "SET @user OFF",
             "SHOW TABLES",
-            "TRUNCATE TABLE x",
             "VACUUM FREEZE my_table",
         ):
             with self.subTest(sql):
@@ -905,3 +995,8 @@ FROM base""",
         with self.assertRaises(UnsupportedError) as ctx:
             unsupported(ErrorLevel.IMMEDIATE)
         self.assertEqual(str(ctx.exception).count(error), 1)
+
+    def test_recursion(self):
+        sql = "1 AND 2 OR 3 AND " * 1000
+        sql += "4"
+        self.assertEqual(len(parse_one(sql).sql()), 17001)
